@@ -19,28 +19,44 @@
 #    under the License.
 
 from __future__ import absolute_import
-import logging
-
 from django.conf import settings
-from manilaclient import client as manila_client
+from horizon import exceptions
+import logging
+from openstack_dashboard.api import base
 import six
 
-from horizon import exceptions
-from horizon.utils.memoized import memoized  # noqa
-from openstack_dashboard.api import base
-from openstack_dashboard.api import nova
-
+from manilaclient import client as manila_client
 
 LOG = logging.getLogger(__name__)
 
 MANILA_UI_USER_AGENT_REPR = "manila_ui_plugin_for_horizon"
-MANILA_VERSION = "2.22"  # requires manilaclient 1.10.0 or newer
+# NOTE(vponomaryov): update version to 2.34 when manilaclient is released with
+# its support. It will allow to show 'availability zones' for share groups.
+MANILA_VERSION = "2.32"  # requires manilaclient 1.13.0 or newer
 MANILA_SERVICE_TYPE = "sharev2"
 
 # API static values
 SHARE_STATE_AVAILABLE = "available"
 DEFAULT_QUOTA_NAME = 'default'
 
+MANILA_QUOTA_FIELDS = {
+    "shares",
+    "share_gigabytes",
+    "share_snapshots",
+    "share_snapshot_gigabytes",
+    "share_networks",
+}
+
+# UI field names do not match data field names returned, have
+# a map to convert them.
+MANILA_QUOTA_FIELDS_DATA_MAP = {
+    "shares": "shares",
+    "share_gigabytes": "gigabytes",
+    "share_snapshots": "snapshots",
+    "share_snapshot_gigabytes": "snapshot_gigabytes",
+    "share_networks": "share_networks"
+}
+
 
 def manilaclient(request):
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
@@ -81,16 +97,21 @@ def share_get(request, share_id):
 
 def share_create(request, size, name, description, proto, snapshot_id=None,
                  metadata=None, share_network=None, share_type=None,
-                 is_public=None, availability_zone=None):
+                 is_public=None, availability_zone=None, share_group_id=None):
     return manilaclient(request).shares.create(
         proto, size, name=name, description=description,
         share_network=share_network, snapshot_id=snapshot_id,
         metadata=metadata, share_type=share_type, is_public=is_public,
-        availability_zone=availability_zone)
+        availability_zone=availability_zone,
+        share_group_id=share_group_id,
+    )
 
 
-def share_delete(request, share_id):
-    return manilaclient(request).shares.delete(share_id)
+def share_delete(request, share_id, share_group_id=None):
+    return manilaclient(request).shares.delete(
+        share_id,
+        share_group_id=share_group_id,
+    )
 
 
 def share_update(request, share_id, name, description, is_public=''):
@@ -140,14 +161,15 @@ def share_manage(request, service_host, protocol, export_path,
 
 
 def migration_start(request, share, dest_host, force_host_assisted_migration,
-                    writable, preserve_metadata, nondisruptive,
-                    new_share_network_id, new_share_type_id):
+                    writable, preserve_metadata, preserve_snapshots,
+                    nondisruptive, new_share_network_id, new_share_type_id):
     return manilaclient(request).shares.migration_start(
         share,
         host=dest_host,
         force_host_assisted_migration=force_host_assisted_migration,
         writable=writable,
         preserve_metadata=preserve_metadata,
+        preserve_snapshots=preserve_snapshots,
         nondisruptive=nondisruptive,
         new_share_network_id=new_share_network_id,
         new_share_type_id=new_share_type_id
@@ -175,6 +197,17 @@ def share_extend(request, share_id, new_size):
     return manilaclient(request).shares.extend(share_id, new_size)
 
 
+def share_revert(request, share, snapshot):
+    """Sends request to revert share to specific snapshot.
+
+    This API available only since 2.27 microversion.
+
+    :param share: Share class instance or share ID
+    :param snapshot: ShareSnapshot class instance or share snapshot ID
+    """
+    return manilaclient(request).shares.revert_to_snapshot(share, snapshot)
+
+
 def share_snapshot_get(request, snapshot_id):
     return manilaclient(request).share_snapshots.get(snapshot_id)
 
@@ -208,6 +241,29 @@ def share_snapshot_delete(request, snapshot_id):
     return manilaclient(request).share_snapshots.delete(snapshot_id)
 
 
+def share_snapshot_allow(request, snapshot_id, access_type, access_to):
+    return manilaclient(request).share_snapshots.allow(
+        snapshot_id, access_type, access_to)
+
+
+def share_snapshot_deny(request, snapshot_id, rule_id):
+    return manilaclient(request).share_snapshots.deny(snapshot_id, rule_id)
+
+
+def share_snapshot_rules_list(request, snapshot_id):
+    return manilaclient(request).share_snapshots.access_list(snapshot_id)
+
+
+def share_snap_export_location_list(request, snapshot):
+    return manilaclient(request).share_snapshot_export_locations.list(
+        snapshot=snapshot)
+
+
+def share_snap_instance_export_location_list(request, snapshot_instance):
+    return manilaclient(request).share_snapshot_export_locations.list(
+        snapshot_instance=snapshot_instance)
+
+
 def share_server_list(request, search_opts=None):
     return manilaclient(request).share_servers.list(search_opts=search_opts)
 
@@ -226,10 +282,10 @@ def share_network_list(request, detailed=False, search_opts=None):
 
 
 def share_network_create(request, neutron_net_id=None, neutron_subnet_id=None,
-                         nova_net_id=None, name=None, description=None):
+                         name=None, description=None):
     return manilaclient(request).share_networks.create(
         neutron_net_id=neutron_net_id, neutron_subnet_id=neutron_subnet_id,
-        nova_net_id=nova_net_id, name=name, description=description)
+        name=name, description=description)
 
 
 def share_network_get(request, share_net_id):
@@ -306,7 +362,20 @@ def tenant_quota_get(request, tenant_id):
     return base.QuotaSet(manilaclient(request).quotas.get(tenant_id))
 
 
+def _map_quota_names_for_update(data):
+    mapping = {
+        'share_gigabytes': 'gigabytes',
+        'share_snapshots': 'snapshots',
+        'share_snapshot_gigabytes': 'snapshot_gigabytes',
+    }
+    for k, v in mapping.items():
+        if k in data:
+            data[v] = data.pop(k)
+    return data
+
+
 def tenant_quota_update(request, tenant_id, **kwargs):
+    _map_quota_names_for_update(kwargs)
     return manilaclient(request).quotas.update(tenant_id, **kwargs)
 
 
@@ -315,6 +384,7 @@ def default_quota_get(request, tenant_id):
 
 
 def default_quota_update(request, **kwargs):
+    _map_quota_names_for_update(kwargs)
     manilaclient(request).quota_classes.update(DEFAULT_QUOTA_NAME, **kwargs)
 
 
@@ -405,14 +475,6 @@ def share_replica_resync(request, replica):
     return manilaclient(request).share_replicas.resync(replica)
 
 
-def share_valid_availability_zones_for_new_replica(request):
-    availability_zones = (
-        set([az.zoneName
-             for az in nova.availability_zone_list(request)])
-    )
-    return availability_zones
-
-
 def tenant_absolute_limits(request):
     limits = manilaclient(request).limits.get().absolute
     limits_dict = {}
@@ -433,13 +495,152 @@ def share_instance_get(request, share_instance_id):
     return manilaclient(request).share_instances.get(share_instance_id)
 
 
-@memoized
-def is_replication_enabled():
-    manila_config = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
-    return manila_config.get('enable_replication', True)
+def availability_zone_list(request):
+    return manilaclient(request).availability_zones.list()
 
 
-@memoized
-def is_migration_enabled():
-    manila_config = getattr(settings, 'OPENSTACK_MANILA_FEATURES', {})
-    return manila_config.get('enable_migration', True)
+def pool_list(request, detailed=False):
+    return manilaclient(request).pools.list(detailed=detailed)
+
+
+# ####### Share Groups # #######
+
+def share_group_create(request, name, description=None,
+                       share_group_type=None,
+                       share_types=None,
+                       share_network=None,
+                       source_share_group_snapshot=None,
+                       availability_zone=None):
+    return manilaclient(request).share_groups.create(
+        name=name,
+        description=description,
+        share_group_type=share_group_type,
+        share_types=share_types,
+        share_network=share_network,
+        source_share_group_snapshot=source_share_group_snapshot,
+        availability_zone=availability_zone,
+    )
+
+
+def share_group_get(request, share_group):
+    return manilaclient(request).share_groups.get(share_group)
+
+
+def share_group_update(request, share_group, name, description):
+    return manilaclient(request).share_groups.update(
+        share_group,
+        name=name,
+        description=description,
+    )
+
+
+def share_group_delete(request, share_group, force=False):
+    return manilaclient(request).share_groups.delete(share_group, force=force)
+
+
+def share_group_reset_state(request, share_group, state):
+    return manilaclient(request).share_groups.reset_state(share_group, state)
+
+
+def share_group_list(request, detailed=True, search_opts=None, sort_key=None,
+                     sort_dir=None):
+    return manilaclient(request).share_groups.list(
+        detailed=detailed,
+        search_opts=search_opts,
+        sort_key=sort_key,
+        sort_dir=sort_dir,
+    )
+
+
+# ####### Share Group Snapshots # #######
+
+def share_group_snapshot_create(request, share_group, name, description=None):
+    return manilaclient(request).share_group_snapshots.create(
+        share_group=share_group,
+        name=name,
+        description=description,
+    )
+
+
+def share_group_snapshot_get(request, share_group_snapshot):
+    return manilaclient(request).share_group_snapshots.get(
+        share_group_snapshot)
+
+
+def share_group_snapshot_update(request, share_group_snapshot, name,
+                                description):
+    return manilaclient(request).share_group_snapshots.update(
+        share_group_snapshot,
+        name=name,
+        description=description,
+    )
+
+
+def share_group_snapshot_delete(request, share_group_snapshot, force=False):
+    return manilaclient(request).share_group_snapshots.delete(
+        share_group_snapshot, force=force)
+
+
+def share_group_snapshot_reset_state(request, share_group_snapshot, state):
+    return manilaclient(request).share_group_snapshots.reset_state(
+        share_group_snapshot, state)
+
+
+def share_group_snapshot_list(request, detailed=True, search_opts=None,
+                              sort_key=None, sort_dir=None):
+    return manilaclient(request).share_group_snapshots.list(
+        detailed=detailed,
+        search_opts=search_opts,
+        sort_key=sort_key,
+        sort_dir=sort_dir,
+    )
+
+
+# ####### Share Group Types # ########
+
+def share_group_type_create(request, name, share_types, is_public=False,
+                            group_specs=None):
+    return manilaclient(request).share_group_types.create(
+        name=name, share_types=share_types, is_public=is_public,
+        group_specs=group_specs)
+
+
+def share_group_type_get(request, share_group_type):
+    return manilaclient(request).share_group_types.get(share_group_type)
+
+
+def share_group_type_list(request, show_all=True):
+    return manilaclient(request).share_group_types.list(show_all=show_all)
+
+
+def share_group_type_delete(request, share_group_type):
+    return manilaclient(request).share_group_types.delete(share_group_type)
+
+
+def share_group_type_access_list(request, share_group_type):
+    return manilaclient(request).share_group_type_access.list(share_group_type)
+
+
+def share_group_type_access_add(request, share_group_type, project):
+    return manilaclient(request).share_group_type_access.add_project_access(
+        share_group_type, project)
+
+
+def share_group_type_access_remove(request, share_group_type, project):
+    return manilaclient(request).share_group_type_access.remove_project_access(
+        share_group_type, project)
+
+
+def share_group_type_set_specs(request, share_group_type, group_specs):
+    return manilaclient(request).share_group_types.get(
+        share_group_type).set_keys(group_specs)
+
+
+def share_group_type_unset_specs(request, share_group_type, keys):
+    return manilaclient(request).share_group_types.get(
+        share_group_type).unset_keys(keys)
+
+
+def share_group_type_get_specs(request, share_group_type):
+    return manilaclient(request).share_group_types.get(
+        share_group_type).get_keys()
